from os import stat
import boto3
import json
from logging import debug, info
from lgw.s3 import upload_file

MAX_LAMBDA_SIZE = 50000000


def deploy_function(
    archive,
    lambda_name,
    handler_name,
    execution_role,
    connection_timeout,
    memory_size,
    runtime,
    s3_bucket,
    s3_key,
    description,
    vpc_subnets,
    vpc_security_groups,
    environment,
    tags,
):

    env = {}
    if environment:
        env = dict(item.split('=') for item in environment.split(';'))

    t = {}
    if tags:
        t = dict(item.split('=') for item in tags.split(';'))

    vpc_config = {}
    if vpc_subnets and vpc_security_groups:
        subnets = vpc_subnets.split(',')
        sec_grps = vpc_security_groups.split(',')
        vpc_config = {'SubnetIds': subnets, 'SecurityGroupIds': sec_grps}

    sz = stat(archive).st_size
    if sz < MAX_LAMBDA_SIZE:
        return deploy_function_from_zip(
            archive,
            lambda_name,
            handler_name,
            execution_role,
            connection_timeout,
            memory_size,
            description,
            vpc_config,
            runtime,
            env,
            t,
        )
    else:
        return deploy_function_from_s3(
            archive,
            lambda_name,
            s3_bucket,
            s3_key,
            handler_name,
            execution_role,
            connection_timeout,
            memory_size,
            description,
            vpc_config,
            runtime,
            env,
            t,
        )


def deploy_function_from_s3(
    archive,
    lambda_name,
    s3_bucket,
    s3_key,
    handler_name,
    execution_role,
    connection_timeout,
    memory_size,
    description=None,
    vpc_config=None,
    runtime='python3.7',
    environment=None,
    tags=None,
):
    upload_file(s3_bucket, s3_key, archive)
    code = {'S3Bucket': s3_bucket, 'S3Key': s3_key}
    return create_or_replace_function(
        lambda_name,
        code,
        handler_name,
        execution_role,
        connection_timeout,
        memory_size,
        description,
        vpc_config,
        runtime,
        environment,
        tags,
    )


def deploy_function_from_zip(
    archive,
    lambda_name,
    handler_name,
    execution_role,
    connection_timeout,
    memory_size,
    description=None,
    vpc_config=None,
    runtime='python3.7',
    environment=None,
    tags=None,
):
    with open(archive, 'rb') as binaryfile:
        zipfile = bytearray(binaryfile.read())
        code = {'ZipFile': zipfile}
        return create_or_replace_function(
            lambda_name,
            code,
            handler_name,
            execution_role,
            connection_timeout,
            memory_size,
            description,
            vpc_config,
            runtime,
            environment,
            tags,
        )


def create_or_replace_function(
    lambda_name,
    code,
    handler_name,
    execution_role,
    connection_timeout,
    memory_size,
    description=None,
    vpc_config=None,
    runtime='python3.7',
    environment=None,
    tags=None,
):
    '''
    Deploys a lambda function to AWS Lambda.  If a function already exists under the given
    `lambda_name` then this will delete it.

    :param lambda_name: Name for the Lambda function
    :param code: Config for location of executable code for the function.
    :param handler_name: Name of the entry point of the function.
    :param execution_role: Name of a role with execute permissions.
    :param vpc_config: Optional VPC config where the function should execute.
                      SubnetIds: [string], SecurityGroupIds: [string].
    :param runtime: Language runtime of the function. Default: python3.7
    :param environment: Environment variables to be available at runtime to the function.
    :param tags: Tags to identify the function.
    :return: ARN of deployed function.
    '''

    delete_function(lambda_name)

    lambda_client = boto3.client('lambda')

    env = {}
    if environment:
        env = {'Variables': environment}

    tracing_config = {'Mode': 'PassThrough'}

    info('Creating a lambda function with name: [%s]' % lambda_name)
    response = lambda_client.create_function(
        FunctionName=lambda_name,
        Runtime=runtime,
        Role=execution_role,
        Handler=handler_name,
        Code=code,
        Description=description,
        Timeout=int(connection_timeout),
        MemorySize=int(memory_size),
        Publish=True,
        VpcConfig=vpc_config,
        Environment=env,
        TracingConfig=tracing_config,
        Tags=tags,
    )

    return response['FunctionArn']


def delete_function(lambda_name):
    '''
    Deletes a lambda function.

    :param lambda_name: Name or ARN of the Lambda function to be deleted.
    :return: None
    '''
    lambda_client = boto3.client('lambda')
    response = lambda_client.list_functions()
    for f in response['Functions']:
        if f['FunctionName'] == lambda_name:
            debug('Found existing function [%s].' % f['FunctionName'])
            lambda_client.delete_function(FunctionName=lambda_name)
            info('Existing function [%s] deleted.' % f['FunctionName'])
            return
    info('No lambda named [%s] found to delete.' % lambda_name)


def invoke_function(lambda_name, payload):
    '''
    Invokes a lambda function.

    :param lambda_name: Name or ARN of the Lambda function to invoke.
    :param payload: File of JSON to send along with invocation.
    :return: None
    '''
    log_type = 'Tail'
    invocation_type = 'RequestResponse'  # or 'Event', or 'DryRun'

    with open(payload, 'r') as file:
        input_payload = bytearray(file)

        lambda_client = boto3.client('lambda')

        res = lambda_client.invoke(
            FunctionName=lambda_name,
            InvocationType=invocation_type,
            LogType=log_type,
            Payload=input_payload,
        )

    return res


def invoke_function(lambda_name):
    '''
    Invokes a lambda function.

    :param lambda_name: Name or ARN of the Lambda function to invoke.
    :return: None
    '''
    log_type = 'Tail'
    invocation_type = 'RequestResponse'  # or 'Event', or 'DryRun'

    lambda_client = boto3.client('lambda')
    res = lambda_client.invoke(
        FunctionName=lambda_name, InvocationType=invocation_type, LogType=log_type
    )

    return res


def get_lambda_info(lambda_name):
    lambda_client = boto3.client('lambda')
    response = lambda_client.get_function(FunctionName=lambda_name)
    lambda_arn = response['Configuration']['FunctionArn']

    sections = lambda_arn.split(':')
    region = sections[3]
    account_id = sections[4]

    # Construct the Lambda function's URI
    lambda_uri = (
        f'arn:aws:apigateway:{region}:lambda:path/2015-03-31/functions/{lambda_arn}/invocations'
    )

    return lambda_arn, lambda_uri, region, account_id


def grant_permission_to_api_resource(api_id, region, account_id, lambda_arn, resource_path):
    '''
    Grant invoke permissions on the Lambda function so it can be called by API Gateway.
    If it exists already then remove so it can be recreated.
    '''
    lambda_client = boto3.client('lambda')
    lambda_name = lambda_arn.split(':')[6]
    statement_id = f'{lambda_name}-invoke'
    action = 'lambda:InvokeFunction'

    info(f'Loading policies related to lambda: {lambda_arn}')
    policy = None
    try:
        policy = lambda_client.get_policy(FunctionName=lambda_arn)
    except lambda_client.exceptions.ResourceNotFoundException:
        info(f'No policy associated with {lambda_arn}')

    if policy and 'Policy' in policy:
        stmts = json.loads(policy['Policy'])
        for stmt in stmts['Statement']:
            if stmt['Action'] == action and stmt['Resource'] == lambda_arn:
                info(f'removing permission [{statement_id}] for lambda: [{lambda_arn}]')
                try:
                    lambda_client.remove_permission(
                        FunctionName=lambda_arn, StatementId=statement_id
                    )
                except lambda_client.exceptions.ResourceNotFoundException:
                    info(
                        f'No permission found for StatementId: {statement_id}, FunctionName: {lambda_arn}'
                    )

    info(f'adding permission [{statement_id}] for lambda: [{lambda_arn}]')
    source_arn = f'arn:aws:execute-api:{region}:{account_id}:{api_id}/*/*/*/'
    lambda_client.add_permission(
        FunctionName=lambda_arn,
        StatementId=statement_id,
        Action=action,
        Principal='apigateway.amazonaws.com',
        SourceArn=source_arn,
    )



import json
from logging import info
import boto3
from botocore.exceptions import ClientError
from lgw.lambda_util import get_lambda_info, grant_permission_to_api_resource


def create_rest_api(api_name, lambda_name, resource_path, deploy_stage, integration_role):
    '''
    Creates & deploys a REST API that proxies to a Lambda function, returning the URL
    pointing to this API.

    :param api_name: Name of the REST API
    :param lambda_name: Name of an existing Lambda function
    :param resource_path: The resource path that points to the lambda.
    :param deploy_stage: The name of the deployment stage.

    :return: URL of API. If error, returns None.
    '''

    api_client = boto3.client('apigateway')

    api_id = create_api_gateway(api_client, api_name)

    (lambda_arn, lambda_uri, region, account_id) = get_lambda_info(lambda_name)

    root_resource_id = get_root_resource_id(api_client, api_id)
    create_method(api_client, api_id, root_resource_id, 'ANY')
    create_lambda_integration(api_client, api_id, root_resource_id, lambda_uri, integration_role)

    child_resource_id = create_resource(api_client, api_id, root_resource_id, resource_path)
    create_method(api_client, api_id, child_resource_id, 'ANY')
    create_lambda_integration(api_client, api_id, child_resource_id, lambda_uri, integration_role)

    deploy_to_stage(api_client, api_id, deploy_stage)

    # grant_permission_to_api_resource(api_id, region, account_id, lambda_arn, resource_path)

    return f'https://{api_id}.execute-api.{region}.amazonaws.com/{deploy_stage}'


def delete_rest_api(api_name):
    api_client = boto3.client('apigateway')
    delete_api_gateway(api_client, api_name)


def deploy_to_stage(api_client, api_id, deploy_stage):
    return api_client.create_deployment(restApiId=api_id, stageName=deploy_stage)


def create_lambda_integration(api_client, api_id, root_resource_id, lambda_uri, role_arn=None):
    '''
    Set the Lambda function as the destination for the ANY method
    Extract the Lambda region and AWS account ID from the Lambda ARN
    ARN format="arn:aws:lambda:REGION:ACCOUNT_ID:function:FUNCTION_NAME"
    '''
    api_client.put_integration(
        restApiId=api_id,
        resourceId=root_resource_id,
        httpMethod='ANY',
        type='AWS_PROXY',
        integrationHttpMethod='POST',
        uri=lambda_uri,
        credentials=role_arn,
    )


def create_method(api_client, api_id, resource_id, http_method):
    try:
        response = api_client.get_method(
            restApiId=api_id, resourceId=resource_id, httpMethod=http_method
        )
        if response and response.get('httpMethod'):
            info(f'{http_method} method already exists for resource {resource_id}')
            return
    except api_client.exceptions.NotFoundException:
        info(f'{http_method} method does not exist for resource {resource_id}, adding it.')

    api_client.put_method(
        resourceId=resource_id, restApiId=api_id, httpMethod=http_method, authorizationType='NONE'
    )

    # Set the content-type of the method response to JSON
    content_type = {'application/json': 'Empty'}
    api_client.put_method_response(
        restApiId=api_id,
        resourceId=resource_id,
        httpMethod=http_method,
        statusCode='200',
        responseModels=content_type,
    )


def create_resource(api_client, api_id, parent_id, resource_path):
    resources = api_client.get_resources(restApiId=api_id)
    if 'items' in resources:
        for resource in resources['items']:
            if resource.get('parentId') == parent_id and resource.get('pathPart') == resource_path:
                info('Found existing resource for %s' % resource['parentId'])
                return resource['id']

    info(f'No existing resource found for {parent_id}/{resource_path}, creating a new one')
    result = api_client.create_resource(
        restApiId=api_id, parentId=parent_id, pathPart=resource_path
    )
    return result['id']


def get_root_resource_id(api_client, api_id):
    result = api_client.get_resources(restApiId=api_id)

    root_id = None
    for item in result['items']:
        if item['path'] == '/':
            root_id = item['id']

    if root_id is None:
        raise ClientError(
            'Could not retrieve the ID of the API root resource using api_id [%s]' % api_id
        )

    return root_id


def delete_api_gateway(api_client, api_name):
    api_id = lookup_api_gateway(api_client, api_name)
    if api_id:
        info(f'Deleting API with ID: {api_id}')
        api_client.delete_rest_api(restApiId=api_id)


def create_api_gateway(api_client, api_name):
    api_id = lookup_api_gateway(api_client, api_name)
    if api_id:
        return api_id
    info(f'No existing API account found for {api_name}, creating it.')
    result = api_client.create_rest_api(name=api_name)
    return result['id']


def lookup_api_gateway(api_client, api_name):
    apis = api_client.get_rest_apis()
    if 'items' in apis:
        for api in apis['items']:
            if api['name'] == api_name:
                info('Found existing API account for %s' % api['name'])
                return api['id']
    info(f'No API gateway found with name {api_name}')
    return None



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['lgw']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.10,<2.0',
 'docker>=4.1,<5.0',
 'docopt>=0.6.2,<0.7.0',
 'everett>=1.0,<2.0']

entry_points = \
{'console_scripts': ['lgw = lgw.main:main']}

setup_kwargs = {
    'name': 'lgw',
    'version': '1.0.1',
    'description': 'Configure an AWS Gateway in front of a Lambda function.',
    'long_description': '# Lambda Gateway\n\nConfigure an AWS Gateway in front of a Lambda function.\n\n## Usage\n\n```\nLambda Gateway.\n\nUsage:\n  lgw gw-deploy [--verbose] --config-file=<cfg>\n  lgw gw-undeploy [--verbose] --config-file=<cfg>\n  lgw domain-add [--verbose] --config-file=<cfg>\n  lgw domain-remove [--verbose] --config-file=<cfg>\n  lgw lambda-deploy [--verbose] --config-file=<cfg> --lambda-file=<zip>\n  lgw lambda-invoke [--verbose] --lambda-name=<name> [--payload=<json>]\n  lgw lambda-delete [--verbose] --lambda-name=<name>\n  lgw lambda-archive [--verbose] --config-file=<cfg>\n\nOptions:\n  -h --help             Show this screen.\n  --version             Show version.\n  --verbose             Enable DEBUG-level logging.\n  --config-file=<cfg>   Override defaults with these settings.\n  --lambda-file=<zip>   Path to zip file with executable lambda code.\n  --lambda-name=<name>  Name of the lambda to invoke or delete.\n  --payload=<json>      Path to a file of type json with data to send with the lambda invocation.\n```\n\n## Configuration Parameters\n\nConfiguration params are read in the following order, with the first read of it overriding subsequent configs:\n\n1. Read from environment.\n2. Read from `.env` file in current folder.\n3. Read from flat file named via `--config-file` CLI param.\n4. Read from `lgw.settings.defaults()`\n\nDefaults are configured in `lgw.settings`.\n\n<table>\n<tr>\n<th>Related Task(s)</th>\n<th>Key</th>\n<th>Description</th>\n<th>Default</th>\n</tr>\n<tr>\n<td><ul><li>All</li></ul></td>\n<td><code>AWS_REGION</code></td>\n<td>AWS region.</td>\n<td><tt>us-east-1</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>gw-deploy</tt></li>\n  <li><tt>gw-undeploy</tt></li>\n  <li><tt>domain-add</tt></li>\n  <li><tt>domain-remove</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_NAME</code></td>\n<td>Name for the created API gateway.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>gw-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_RESOURCE_PATH</code></td>\n<td>Resource path for the API. By default it\'s a greedy path to proxy all requests.</td>\n<td><tt>{proxy+}</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>gw-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_DEPLOY_STAGE</code></td>\n<td>Name for the stage that the API gets deployed to. E.g. "production"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>gw-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_LAMBDA_INTEGRATION_ROLE</code></td>\n<td>ARN of a role that grants permission to the API gateway to invoke a lambda.  Should have <tt>AmazonAPIGatewayPushToCloudWatchLogs</tt> and <tt>AWSLambdaRole</tt> managed roles as permissions, and <tt>apigateway.amazonaws.com</tt> as a trusted entity.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>domain-add</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_DOMAIN_NAME</code></td>\n<td>A domain name configured in Route 53 that the API gateway can be mapped to.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>domain-add</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_BASE_PATH</code></td>\n<td>Base path mapping to connect the domain name\'s CF distribution to the gateway.</td>\n<td><tt>(none)</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>domain-add</tt></li>\n</ul>\n</td>\n<td><code>AWS_API_DOMAIN_WAIT_UNTIL_AVAILABLE</code></td>\n<td>Waits until the custom domain name has been created.</td>\n<td>true, set to undefined to disable.</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>domain-add</tt></li>\n</ul>\n</td>\n<td><code>AWS_ACM_CERTIFICATE_ARN</code></td>\n<td>ARN of an HTTPS certificate to use for securing API requests.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>gw-deploy</tt></li>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_NAME</code></td>\n<td>Name for the created Lambda.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_DESCRIPTION</code></td>\n<td>Description for the created Lambda</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_HANDLER</code></td>\n<td>Name of the handler function. e.g. "module.function"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_RUNTIME</code></td>\n<td>Lambda runtime environment.</td>\n<td><tt>python3.7</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_CONNECTION_TIMEOUT</code></td>\n<td>Connection timeout in seconds.</td>\n<td><tt>30</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_MEMORY_SIZE</code></td>\n<td>Amount of memory to allocate to the Lambda.</td>\n<td><tt>3000</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_BUCKET</code></td>\n<td>S3 bucket to store lambda if zip file exceeds maximum upload size.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_KEY</code></td>\n<td>Key of the lambda archive in the configured bucket.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_EXECUTION_ROLE_ARN</code></td>\n<td>ARN of a role with permissions to execute the Lambda.  Should have <tt>AWSXrayWriteOnlyAccess</tt> and <tt>AWSLambdaBasicExecutionRole</tt> managed roles as permissions, and <tt>lambda.amazonaws.com</tt> as a trusted entity.</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_VPC_SUBNETS</code></td>\n<td>List of subnets that the Lambda should run in. Format: "subnetA,subnetB,subnetC,...</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_VPC_SECURITY_GROUPS</code></td>\n<td>List of security groups that control the Lambda\'s access. Format: "<tt>secgrpA,secgrpB,secgrpC,...</tt>"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ENVIRONMENT</code></td>\n<td>Variables to inject into the Lambda\'s environment.  Format: "<tt>varA=valA;varB=valB;...</tt>"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-deploy</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_TAGS</code></td>\n<td>List of tags to categorize this Lambda.  Format: "<tt>tagA=valA;tagB=valB;...</tt>"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-archive</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_CONTEXT_DIR</code></td>\n<td>Root directory of the project that will provide files to be copied into the Docker image.  If the directory ends with a trailing slash, then the root of the context will be the contents of the directory; otherwise the leaf directory will be at the root of the context.</td>\n<td><tt>.</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-archive</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_BUNDLE_DIR</code></td>\n<td>Destination directory to write Lambda archive zipfile. </td>\n<td><tt>./build</tt></td>\n</tr>\n<tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-archive</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_BUNDLE_NAME</code></td>\n<td>Filename of Lambda archive zipfile. </td>\n<td><tt>lambda-bundle.zip</tt></td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-archive</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_ADDL_FILES</code></td>\n<td>List of 2-tuples of files to copy into the context directory from the local computer. Format: "<tt>srcA,desA;srcB=desB;srcC=desC;...</tt>"</td>\n<td>N/A</td>\n</tr>\n<tr>\n<td>\n<ul>\n  <li><tt>lambda-archive</tt></li>\n</ul>\n</td>\n<td><code>AWS_LAMBDA_ARCHIVE_ADDL_PACKAGES</code></td>\n<td>List of <tt>yum</tt> packages to install in the Docker image.  Format: "<tt>packageA,packageB,packageC,...</tt>"</td>\n<td>\nDefault installed by this script:\n<ul>\n<li><tt>gcc</tt></li>\n<li><tt>openssl-devel</tt></li>\n<li><tt>bzip2-devel</tt></li>\n<li><tt>libffi-devel</tt></li>\n<li><tt>python37-pip</tt></li>\n</ul>\n</td>\n</tr>\n</table>\n\n\n## Releasing\n\n```\ngit flow release start x.y.z\n# bump version\nvi pyproject.toml lgw/version.py\ngit add pyproject.toml lgw/version.py\ngit commit -m \'bump version\'\ndephell deps convert\npoetry publish --build\n```\n\n## Info\n\n[![GitHub watchers](https://img.shields.io/github/watchers/badges/shields.svg?style=social&label=Watch&style=flat-square)]()\n[![PyPi License](https://img.shields.io/pypi/l/lgw?color=blue)]()\n[![PyPi](https://img.shields.io/pypi/v/lgw.svg?style=flat-square)]()\n',
    'author': 'Edward Q. Bridges',
    'author_email': 'ebridges@roja.cc',
    'url': 'https://github.com/ebridges/lgw',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from logging import info, warn
import boto3


def update_dns_a_record(domain_name, alias_target_dns_name):
    '''
    Updates the A record for the given domain name with a new alias target.
    Assumes that the hosted zone that hosts the domain name is public, and that
    that the domain name is the apex for this hosted zone.
    '''
    r53_client = boto3.client('route53')

    zone_id = get_hosted_zone_id_for_domain(r53_client, domain_name)
    record_set = {
        'Name': domain_name,
        'Type': 'A',
        'AliasTarget': {
            'HostedZoneId': 'Z2FDTNDATAQYW2',  # This is a magic value that means 'CloudFront'
            'DNSName': alias_target_dns_name,
            'EvaluateTargetHealth': False,
        },
    }

    response = r53_client.change_resource_record_sets(
        HostedZoneId=zone_id,
        ChangeBatch={'Changes': [{'Action': 'UPSERT', 'ResourceRecordSet': record_set}]},
    )

    if response:
        change_info = response.get('ChangeInfo')
        info('Resource record change submitted: status of change is: [%s]' % change_info['Status'])
        if change_info['Status'] == 'PENDING':
            waiter = r53_client.get_waiter('resource_record_sets_changed')
            waiter.wait(Id=change_info['Id'], WaiterConfig={'Delay': 30, 'MaxAttempts': 60})
        return change_info['Id']
    else:
        warn('Empty response returned from record change submission.')
        return None


def get_hosted_zone_id_for_domain(route53_client, domain_name):
    '''
    List public zones -- more robust implementation linked below that considers
    - larger number of hosted zones
    - more diverse pool of domain names
    - non-public hosted zones
    cf.: https://github.com/Miserlou/Zappa/blob/master/zappa/core.py#L3087
    '''
    public_zones = []

    zones = route53_client.list_hosted_zones(MaxItems='25')

    for hosted_zone in zones['HostedZones']:
        if not hosted_zone['Config']['PrivateZone']:
            public_zones += zones['HostedZones']

    for zone in public_zones:
        if zone['Name'][:-1] == domain_name:
            return zone['Id']

    return None



def defaults():
  return {
    'aws_region': 'us-east-1',
    'aws_api_name': '',
    'aws_api_resource_path': '{proxy+}',
    'aws_api_deploy_stage': '',
    'aws_api_lambda_integration_role': '',
    'aws_api_domain_name': '',
    'aws_api_base_path': '(none)',
    'aws_api_domain_wait_until_available': 'true',
    'aws_acm_certificate_arn': '',
    'aws_lambda_name': '',
    'aws_lambda_description': '',
    'aws_lambda_handler': '',
    'aws_lambda_runtime': 'python3.7',
    'aws_lambda_connection_timeout': 30,
    'aws_lambda_memory_size': 3000,
    'aws_lambda_archive_bucket': '',
    'aws_lambda_archive_key': '',
    'aws_lambda_execution_role_arn': '',
    'aws_lambda_vpc_subnets': '',
    'aws_lambda_vpc_security_groups': '',
    'aws_lambda_environment': '',
    'aws_lambda_tags': '',
    'aws_lambda_archive_context_dir': '.',
    'aws_lambda_archive_bundle_dir': './build',
    'aws_lambda_archive_bundle_name': 'lambda-bundle.zip',
    'aws_lambda_archive_addl_files': '',
    'aws_lambda_archive_addl_packages': '',
  }
